"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var BaseInteceptor = /** @class */ (function () {
    function BaseInteceptor(mocker) {
        this.mocker = mocker;
        this.global = BaseInteceptor.getGlobal();
    }
    /**
     * Setup request mocker.
     */
    BaseInteceptor.setup = function (mocker) {
        return new this(mocker);
    };
    /**
     * return global variable
     */
    BaseInteceptor.getGlobal = function () {
        if (typeof window !== 'undefined') {
            return window;
        }
        else if (typeof global !== 'undefined') {
            return global;
        }
        else if (typeof self !== 'undefined') {
            return self;
        }
        throw new Error('Detect global variable error');
    };
    /**
     * Check whether the specified request url matchs a defined mock item.
     * If a match were found, return mock meta information, otherwise a null is returned.
     * @param {string} reqUrl
     * @param {string} reqMethod
     */
    BaseInteceptor.prototype.matchMockRequest = function (reqUrl, reqMethod) {
        return this.mocker.matchMockItem(reqUrl, reqMethod);
    };
    /**
     * Get query parameters from the specified request url.
     * @param {string} reqUrl
     */
    BaseInteceptor.prototype.getQuery = function (reqUrl) {
        return /\?/.test(reqUrl)
            ? reqUrl
                .replace(/.*?\?/g, '') // no protocol, domain and path
                .replace(/#.*$/g, '') // no hash tag
                .split('&')
                .reduce(function (res, item) {
                var _a = item.split('='), k = _a[0], v = _a[1];
                res[k] = (v || '').trim();
                return res;
            }, {})
            : {};
    };
    return BaseInteceptor;
}());
exports.default = BaseInteceptor;
//# sourceMappingURL=base.js.map