"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var base_1 = __importDefault(require("./base"));
var WxRequestInterceptor = /** @class */ (function (_super) {
    __extends(WxRequestInterceptor, _super);
    function WxRequestInterceptor(mocker) {
        var _this = _super.call(this, mocker) || this;
        if (WxRequestInterceptor.instance) {
            return WxRequestInterceptor.instance;
        }
        WxRequestInterceptor.instance = _this;
        _this.wxRequest = _this.global.wx.request.bind(wx);
        _this.intercept();
        return _this;
    }
    /**
     * Setup request mocker for unit test.
     */
    WxRequestInterceptor.setupForUnitTest = function (mocker) {
        var global = base_1.default.getGlobal();
        global.wx = global.wx || {};
        if (!global.wx.request) {
            global.wx.request = function () { };
        }
        return new WxRequestInterceptor(mocker);
    };
    /**
     * https://developers.weixin.qq.com/miniprogram/dev/api/network/request/wx.request.html
     * Intercept wx.request object.
     */
    WxRequestInterceptor.prototype.intercept = function () {
        var _this = this;
        Object.defineProperty(wx, 'request', {
            configurable: true,
            enumerable: true,
            writable: true,
            value: function (requestInfo) {
                if (!requestInfo || !requestInfo.url) {
                    return;
                }
                var match = _this.matchMockRequest(requestInfo.url, requestInfo.method);
                if (match) {
                    requestInfo.query = _this.getQuery(requestInfo.url);
                    _this.doMockRequest(match, requestInfo);
                }
                else {
                    _this.wxRequest(requestInfo); // fallback to original wx.request
                }
            }
        });
        return this;
    };
    /**
     * Make mock request.
     * @param {MockItemInfo} match
     * @param {WxRequestInfo} requestInfo
     */
    WxRequestInterceptor.prototype.doMockRequest = function (match, requestInfo) {
        var _this = this;
        if (match.file) {
            // To avoid "Critical dependency: the request of a dependency is an expression" error
            Promise.resolve().then(function () { return __importStar(require(process.env.HRM_MOCK_DIR + "/" + match.file)); }).then(function (mock) {
                var mockResponse = _this.getMockResponse(mock.default, match, requestInfo);
                _this.doMockResponse(mockResponse, match, requestInfo);
            });
            return;
        }
        var mockResponse = this.getMockResponse(match.response, match, requestInfo);
        this.doMockResponse(mockResponse, match, requestInfo);
    };
    /**
     * Make mock response.
     * @param {any} response
     * @param {MockItemInfo} match
     * @param {WxRequestInfo} requestInfo
     */
    WxRequestInterceptor.prototype.doMockResponse = function (response, match, requestInfo) {
        var _this = this;
        if (match.delay && match.delay > 0) {
            setTimeout(function () {
                _this.doCompleteCallbacks(requestInfo, response);
            }, +match.delay);
        }
        else {
            this.doCompleteCallbacks(requestInfo, response);
        }
    };
    /**
     * Format mock data to fit wx.request callbacks.
     * @param {any} mockResponseConfig
     * @param {MockItemInfo} match
     * @param {WxRequestInfo} requestInfo
     */
    WxRequestInterceptor.prototype.getMockResponse = function (mockResponseConfig, match, requestInfo) {
        var data = typeof mockResponseConfig === 'function' ? mockResponseConfig(requestInfo) : mockResponseConfig;
        // https://developers.weixin.qq.com/miniprogram/dev/api/network/request/wx.request.html
        return {
            data: data,
            statusCode: match.status || 200,
            header: __assign(__assign({}, match.header), { 'is-mock': 'yes' }),
            cookies: [],
            profile: {},
        };
    };
    /**
     * Call some necessary callbacks if specified.
     * @param {WxRequestInfo} requestInfo
     * @param {any} response
     */
    WxRequestInterceptor.prototype.doCompleteCallbacks = function (requestInfo, response) {
        if (typeof requestInfo.success === 'function') {
            requestInfo.success(response);
        }
        if (typeof requestInfo.complete === 'function') {
            requestInfo.complete(response);
        }
    };
    return WxRequestInterceptor;
}(base_1.default));
exports.default = WxRequestInterceptor;
//# sourceMappingURL=wx-request.js.map